/*
File: 		Arduino_Cold_Room_Control
Author:		Gordon D. Dick, P. Eng. (retired)

    Operating Mode is established in Setup: If none of the 5 LHS buttons are pressed out of reset, the Mode will be Normal Mode.
                                            If ANY of the LHS button are pressed out of reset, the Mode will be Testing Mode.
    
    The Testing Mode range of temperatures is 18C to 26C. This range of temps was used in the PIC version. Maybe the choice of range could've been better.
        The flapper valve position (for testing Normal Mode) is given by:  gucDesiredPosition = 25 * (uiTemp >> 1) - 450
 
    The Normal Mode range of temperatures (for testing) is 18C to 23C. This range of temps is easily tested at room temp with just a finger to raise the temperature.
        The flapper valve position (for testing Normal Mode) is given by:  gucDesiredPosition = 40 * ucTemp - 720
    The Normal Mode range of temperatures (for a typical Cold Room) is 2C to 10C.                    
        The flapper valve position (for a typical Cold Room) is given by:  gucDesiredPosition = 25 * ucTemp - 50

*/







/*********************************************************************************************************************************************************************************
      This code was developed using Arduino 1.6.3 on a Arduino Uno R3 with a 16 MHz xtal using an Atmel ATmega 328P-PU and a LinkSprite 2x16 LCD shield.
*********************************************************************************************************************************************************************************/


/*           I/O Used

Serial communication:
  Arduino pin D0 is used for RXD, PD0
  Arduino pin D1 is used for TXD, PD1

LCD 16x2 LinkSprite Board:
  Arduino pin D8 is used for RS, PB0
  Arduino pin D9 is used for E (active high CS for LCD), PB1
  Arduino pins D4:D7 are used for 4 bit data bus for LCD, PD4:PD7
  Arduino pin D10 is used for LCD Backlight, PB2

Dallas DS18B20:  
  Arduino pin D2 is used for the One Wire connection, PD2

EasyDriver:  
  Arduino pin D13 is used for EasyDriver Step, PB5
  Arduino pin D12 is used for EasyDriver Direction, PB4
  Arduino pin D11 is used for EasyDriver CS*, PB3
  
Home Switch:
  Arduino pin D3 is used for the Home Switch, PD3

*/
/*************************************************************************************************************************************************
      Includes
*************************************************************************************************************************************************/
    #include <LiquidCrystal.h>
    LiquidCrystal lcd(8,9,4,5,6,7);
    #include <OneWire.h>
    OneWire  ds( 2 );                                    // on pin 2 (a 4.7K resistor is necessary)


/*************************************************************************************************************************************************
      Defines
*************************************************************************************************************************************************/
    #define TTmax 26							//Test mode max temp ( 26C )
    #define TTmin 18							//Test mode min temp ( 18C )
//    #define NTmax 20							//Normal mode max temp ( 10C )
//    #define NTmin 4							//Normal mode min temp ( 2C )
    #define NTmax 46							//Normal mode max temp ( 23C )
    #define NTmin 36							//Normal mode min temp ( 18C )
    
    #define HomePin 3                                                   //PD3 is Home input
    #define EasySelPin 11                                               //PB3 is EasyDriver CS*
    #define DirPin 12                                                   //PB4 is EasyDriver Direction, 0 = CW
    #define StepPin 13                                                  //PB5 is EasyDriver Step
 //   #define BackLightPin 10                                             //PB2 is LinkSprite Back Light, default is Back Light ON


/*************************************************************************************************************************************************
      Global Constants
*************************************************************************************************************************************************/


/*************************************************************************************************************************************************
    GLOBAL VARIABLES
*************************************************************************************************************************************************/
    volatile byte gucTicCount = 0;
    volatile boolean gbTicFlag = 0;
    volatile byte gucPass = 0;
    volatile byte gucMins = 0;
    volatile byte gucActualPosition = 0;
    volatile byte gucDesiredPosition = 0;
    volatile boolean gbModeFlag = true;


    void setup()
    {
      byte cTemp = 0;
      int iTemp = 0;
//Establish directions for the EasyDriver pins and the Home input.  
      pinMode( EasySelPin, OUTPUT );
      pinMode( DirPin, OUTPUT );
      pinMode( StepPin, OUTPUT );
      pinMode( HomePin, INPUT_PULLUP );
      pinMode( BackLightPin, OUTPUT );
      
 //     digitalWrite( BackLightPin, HIGH );
      
      delay( 1000 );                                              //makesure power is up for the lcd
      lcd.begin(20,2);
      delay(500);
  
      lcd.setCursor( 0, 0 );                                      //show splash screens for 3 sec each
      lcd.print( "  Uno Cold Room " );    
      lcd.setCursor( 0, 1 );
      lcd.print( "   Control by   " );
      delay( 3000 );
      lcd.setCursor( 0, 0 );
      lcd.print( " G. D. Dick, EE " );
      lcd.setCursor( 0, 1 );
      lcd.print( "Rev 1.0 Jan 2016" );
      delay( 3000 );
      
      lcd.setCursor( 0, 0 );
      lcd.print( " Motor is homing" );
      lcd.setCursor( 0, 1 );
      lcd.print( "                " );
      
     
//Need to Home the flapper valve before we start.
    digitalWrite( EasySelPin, LOW );				//enable EasyDriver
    digitalWrite( DirPin, LOW );				//establish CW direction
    digitalWrite( StepPin, LOW );                               //start with the Step pin low
    
    while( digitalRead( HomePin ) && (cTemp < 250) )		//Home switch is normally open
	{
		digitalWrite( StepPin, HIGH );
		digitalWrite( StepPin, LOW );			//generate a step pulse
		delay( 20 );					//pause a little
		cTemp++;					//		and count a step
	}
	if( cTemp >= 250 )
	{
		lcd.setCursor( 0,1 ); 
                lcd.print( " Homing failed! " );		//announce " Homing failed! "
                digitalWrite( EasySelPin, HIGH );		//disable EasyDriver
		while(1);					//lock the micro up here if Homing failed
	}
        digitalWrite( EasySelPin, HIGH );			//disable EasyDriver
	gucActualPosition = 0;					//this position is now 0

//When Homing is sucessful need to decide what Mode to operate in.
      iTemp = analogRead( A0 );
      if( iTemp > 1000 )
      {
          gbModeFlag = true;                                    //no buttons are pressed, so default to Normal Mode
          lcd.setCursor( 0, 0 );
          lcd.print( "No button press " );                      //show Normal
          lcd.setCursor( 0, 1 );
          lcd.print( "so Mode = NORMAL" );                      //            Mode
      }
      else
      {
          gbModeFlag = false;                                   //a button is pressed, so run in Test Mode
          lcd.setCursor( 0, 0 );
          lcd.print( "  Button press  " );                      //show Test
          lcd.setCursor( 0, 1 );
          lcd.print( " so Mode = TEST " );                      //            Mode
      }
      delay( 3000 );                                            //show the Mode info for 3 sec

//When Homing is sucessful need to paint the operational screen.
          lcd.setCursor( 0, 0 );
          lcd.print( "OldTmp: xx.x    " );                      //show Old Temperature
          lcd.setCursor( 0, 1 );
          lcd.print( "Tmp: xx.x  P:xxx" );                      //show current Temperature and Flapper position
  
//  set up Timer 1
      TCCR1A = 0;                                                 // normal timer operation
      TCCR1B = bit(WGM12) | bit(CS10) | bit (CS12);               // CTC, scale to clock / 1024
      OCR1A =  249;                                               // compare A register value to give 16 ms int rate
      TIMSK1 = bit (OCIE1A);                                      // interrupt on Compare A Match
    } // end of setup




void loop(void)
{
    byte ucTemp = 0;
    byte ucOldTemp = 0;
    int uiTemp = 0;
    int uiOldTemp = 0;
    byte ucPassOld = 0;
    byte ucMinsOld = 0;
    byte ucTempBuffer[6] = { 0 };
    byte ucPosBuffer[4] = { 0 };
    byte ucTmaxCount = 0;
    byte ucTminCount = 0;
    int uiCircBuff[8] = { 0 };
    byte ucCircBuffIndex = 0;
    int uiCircBuffSum = 0;
    byte ucModeFlag = 0;
  
//The main loop begins here.
  while(1)
  {
    while( !gbTicFlag );				        //hang the processor here until a new tick arrives
	gbTicFlag = false;			        	//clear the flag for next time

//****************************** TEST Mode starts here
	if( !gbModeFlag )                                       //a false Mode Flag indicates run in Test Mode
        {
//This code executes once per second.
	    if( gucPass != ucPassOld )
	    {
	        lcd.setCursor( 13, 1 );
                lcd.print( "   " );                              //erase
                lcd.setCursor( 13, 1 );
                lcd.print( gucActualPosition, DEC );             //show actual position
                uiTemp = ReadOneWire();                          //get a new temperature sample
                ucTemp = uiTemp >> 1;                            //just use the whole number portion here
		if( ucTemp < TTmin )				 //make sure the temperature used is bounded
                    ucTemp = TTmin;
		if( ucTemp > TTmax )
		    ucTemp = TTmax;
                lcd.setCursor(5, 1);
                lcd.print( "  " );                               //erase
                lcd.setCursor(5, 1);
                lcd.print( ucTemp, DEC );                        //show the integer portion of temperature
                lcd.print( "." );                                //put in a decimal point
		if( uiTemp & 0x0001 )			         //show the fractional part
		    lcd.print( "5" );
		else
		    lcd.print( "0" );
                
		if( ucTemp >= (ucOldTemp + 2) )
		{
		    gucDesiredPosition = 25 * (uiTemp >> 1) - 450;	//calculate a new desired position
		    ucOldTemp = ucTemp;					//save a copy of the Old Temp for next pass
                    lcd.setCursor( 8, 0 );
                    lcd.print( "    " );
                    lcd.setCursor( 8, 0 );
                    lcd.print( ucOldTemp, DEC );                        //show the Old Temp
                    
		}//end of if( ucTemp >= (ucOldTemp + 2) )

//If the temperature is at TTmax for 5 consecutive passes, open the flap to max.
		if( ucTemp >= TTmax )
		{
		    ucTmaxCount++;
		    if( ucTmaxCount >= 4 )				//5 consecutive passes yet?
		    {
			gucDesiredPosition = 200;			//If Yes, force flap fully open.
			ucTmaxCount = 4;				//don't count passes past 4
		     }
		}	
		else
		    ucTmaxCount = 0;

		if( ucTemp <= (ucOldTemp - 3) )
		{
		    gucDesiredPosition = 25 * (uiTemp >> 1) - 450;	//calculate a new desired position
		    ucOldTemp = ucTemp;					//save a copy of the Old Temp for next pass
                    lcd.setCursor( 8, 0 );
                    lcd.print( "    " );
                    lcd.setCursor( 8, 0 );
                    lcd.print( ucOldTemp, DEC );                        //show the Old Temp
		 }//end of if( ucTemp <= (uiOldTemp - 3) )

//If the temperature is at TTmin for 5 passes, close the flap.
		if( ucTemp <= TTmin )
		{
		    ucTminCount++;
		    if( ucTminCount >= 4 )				//5 consecutive passes yet?
		    {
		        gucDesiredPosition = 0;				//If Yes, force flap closed.
			ucTminCount = 4;				//don't count passes past 4
		    }
		}	
		else
		    ucTminCount = 0;

		ucPassOld = gucPass;					//update the pass count
                lcd.setCursor( 8, 0 );
		Blink( gucPass );					//			and blink the symbol

	}//end of if( ucPass != ucPassOld )
  }//end of if( !ucModeFlag )

//*********************** NORMAL Mode starts here
  else
  {
//This code executes once per second.

            if( gucPass != ucPassOld )
            {
	        lcd.setCursor( 13, 1 );
                lcd.print( "   " );                              //erase
                lcd.setCursor( 13, 1 );
                lcd.print( gucActualPosition, DEC );             //show actual position
                lcd.setCursor( 14, 0 );
                lcd.print( "  " );                               //erase
                lcd.setCursor( 14, 0 );
                lcd.print( gucPass );                            //show pass count
	    }

//This code executes once per minute.
	    if( gucMins != ucMinsOld )
	    {
	        uiTemp = ReadOneWire();				//get a new temperature sample
 		if( uiTemp < NTmin )				 //make sure the temperature used is bounded
                    uiTemp = NTmin;
		if( uiTemp > NTmax )
		    uiTemp = NTmax;
                ucTemp = uiTemp >> 1;                            //get the integer portion
                lcd.setCursor(5, 1);
                lcd.print( "    " );                            //erase
                lcd.setCursor(5, 1);
                lcd.print( ucTemp, DEC );                       //show the integer portion of temperature
                lcd.print( "." );                                //put in a decimal point
		if( uiTemp & 0x0001 )			         //show the fractional part
		    lcd.print( "5" );
		else
		    lcd.print( "0" );

		if( ucTemp >= (ucOldTemp + 1) )
		{
//		    gucDesiredPosition = 25 * ucTemp - 50;	        //calculate a new desired position
		    gucDesiredPosition = 40 * ucTemp - 720;	        //calculate a new desired position
		    uiOldTemp = uiTemp;					//save a copy of the Old Temp for next pass
                    lcd.setCursor( 8, 0 );
                    lcd.print( "    " );
                    lcd.setCursor( 8, 0 );
                    lcd.print( uiOldTemp >> 1, DEC );                //show the integer part of Old Temp
                    lcd.print( "." );                                //put in a decimal point
		    if( uiOldTemp & 0x0001 )			     //show the fractional part
		        lcd.print( "5" );
		    else
		        lcd.print( "0" );
                   
		}//end of if( ucTemp >= (ucOldTemp + 1) )

//If the temperature is at TTmax for 5 consecutive passes, open the flap to max.
		if( uiTemp >= NTmax )
		{
		    ucTmaxCount++;
		    if( ucTmaxCount >= 4 )				//5 consecutive passes yet?
		    {
			gucDesiredPosition = 200;			//If Yes, force flap fully open.
			ucTmaxCount = 4;				//don't count passes past 4
		     }
		}	
		else
		    ucTmaxCount = 0;

		if( ucTemp <= (ucOldTemp - 2) )
		{
//		    gucDesiredPosition = 25 * ucTemp - 50;	        //calculate a new desired position
		    gucDesiredPosition = 40 * ucTemp - 720;	        //calculate a new desired position
		    uiOldTemp = uiTemp;					//save a copy of this temp for next pass
                    lcd.setCursor( 8, 0 );
                    lcd.print( "    " );
                    lcd.setCursor( 8, 0 );
                    lcd.print( uiOldTemp >> 1, DEC );                //show the Old Temp
                    lcd.print( "." );                                //put in a decimal point
		    if( uiOldTemp & 0x0001 )			     //show the fractional part
		        lcd.print( "5" );
		    else
		        lcd.print( "0" );
		 }//end of if( ucTemp <= (uiOldTemp - 2) )
 
//If the temperature is at NTmin for 5 passes, close the flap.
		if( uiTemp <= NTmin )
		{
		    ucTminCount++;
		    if( ucTminCount >= 4 )				//5 consecutive passes yet?
		    {
		        gucDesiredPosition = 0;				//If Yes, force flap closed.
			ucTminCount = 4;				//don't count passes past 4
		    }
		}	
		else
		    ucTminCount = 0;

		ucMinsOld = gucMins;					//update the pass count

 	    }//end of if( gucMins != ucMinsOld )
    }//end of else( !ucModeFlag )
  }//end of while(1)
}//end of loop()


/*************************************************************************************************************************************************
      ISRs
*************************************************************************************************************************************************/

ISR(TIMER1_COMPA_vect)
{
	gucTicCount ++;
	gbTicFlag = true;          			    //show that a tick has ocurred
	if( gucTicCount >= 62 )				    // 61 * 16.384 ms = 0.999 sec
	{
	    gucTicCount = 1;
	    gucPass++;
	}
//	if( gucPass > 59 )
	if( gucPass > 9 )                                      //for testing
	{
	    gucMins++;
	    gucPass = 0;
	}

	if( gucDesiredPosition > gucActualPosition )		//If Yes, need to go CCW.
	{
            digitalWrite( DirPin, HIGH );			//establish CCW direction
            digitalWrite( EasySelPin, LOW );			//enable EasyDriver
            delay( 5 );
	    digitalWrite( StepPin, HIGH );
	    digitalWrite( StepPin, LOW );			//generate a step pulse
            digitalWrite( EasySelPin, HIGH );			//disable EasyDriver
            gucActualPosition++;
	}
	if( gucDesiredPosition < gucActualPosition )		//If Yes, need to go CW.
	{
            digitalWrite( DirPin, LOW );			//establish CW direction
            digitalWrite( EasySelPin, LOW );			//enable EasyDriver
            delay( 5 );
	    digitalWrite( StepPin, HIGH );
	    digitalWrite( StepPin, LOW );			//generate a step pulse
            digitalWrite( EasySelPin, HIGH );			//disable EasyDriver
            gucActualPosition--;
	}
}


/*************************************************************************************************************************************************
      Functions
*************************************************************************************************************************************************/


/*************************************************************************************************************************************************
      ReadOneWire()
*************************************************************************************************************************************************/
byte ReadOneWire( void )
{
    byte ucRawMSB, ucRawLSB;
    byte ucTemperature;
    int uiTemperature;
  
      ds.reset();
      ds.write( 0xCC );                                          //skip ROM
      ds.write( 0x44 );                                          // start conversion
      delay( 800);                                               // maybe 750ms is enough, maybe not
      ds.reset();
      ds.write( 0xCC );                                          //skip ROM
      ds.write( 0xBE );                                          // Read Scratchpad
      ucRawLSB = ds.read();                                       //need to read 2 bytes
      ucRawMSB = ds.read();
      uiTemperature = ( ucRawMSB << 8 ) | ucRawLSB;                 //this is the 16 bit temperature raw value
      uiTemperature = uiTemperature >> 3;                        // 9 bit resolution
//      ucTemperature = uiTemperature & 0x00FF;
 return uiTemperature;
//****************************Need to check for negative temps
}

/*************************************************************************************************************************************************
	Blink():	Blinks a colon symbol on the lcd.	
*************************************************************************************************************************************************/
void Blink( char ucColon )
{
	if(( ucColon & 0x01 ) == 0 )				//blank off all but LSBit
		lcd.print( ":" );				//		and show the colon
	else
		lcd.print( " " );				//				or show a blank
}



